use mybin::errors;

use clap::{Arg, App};
use log::{info, trace, warn};

use std::fs;

#[derive(Debug)]
struct Settings {
    src: String,
    dst: String,
}

fn parse_args() -> Result<Settings, Box<dyn std::error::Error>> {
    let matches = App::new("Given files in a directory, determine values and store results in a directory")
        .version("1.0")
        .author("Lukas Prokop <lukas.prokop@iaik.tugraz.at>")
        .arg(Arg::with_name("src")
            .value_name("SRC")
            .help("a directory of source files")
            .takes_value(true))
        .arg(Arg::with_name("dst")
            .value_name("DST")
            .help("a directory to write files to")
            .takes_value(true))
        .get_matches();

    let dir1: &str = (&matches).value_of("src")
        .ok_or(errors::ArgError::new_err("argument #1 'src' must be a directory path"))?;
    let dir2: &str = (&matches).value_of("dst")
        .ok_or(errors::ArgError::new_err("argument #2 'dst' must be a directory path"))?;

    Ok(Settings { src: dir1.to_string(), dst: dir2.to_string() })
}

fn run(settings: &Settings) -> Result<(), Box<dyn std::error::Error>> {
    for path in fs::read_dir(&settings.src)? {
        let filepath = path?;
        info!("process {:?}", (&filepath).path().display());
    }

    warn!("I am about to finish");
    trace!(
        target: "exec1", "src is {} and dst is {}",
        settings.src, settings.dst
    );

    Ok(())
}

fn main() {
    let settings = match parse_args() {
        Ok(s) => s,
        Err(e) => { eprintln!("{}", e); std::process::exit(1) },
    };

    match run(&settings) {
        Ok(_) => {},
        Err(e) => { eprintln!("error: {}", e); std::process::exit(2) },
    };
}